classdef batteries < handle
    % Battery class
    
    properties
        C
        c
        Ialpha
        Tbeta
        QB0
        U0
        R
        T0
        T50
        cpB
        mB
        hB
    end
    
    methods
        function obj = batteries(battery)
            % constructs the class
            % 
            obj.c      = [battery('c[0]'),battery('c[1]'),battery('c[2]'),battery('c[3]'),battery('c[4]'),battery('c[5]')];
            obj.C      = battery('1C');
            obj.Ialpha = battery('I[alpha]');
            obj.QB0    = battery('Q[B0]');
            obj.U0     = battery('U[0]');
            obj.R      = [battery('R[1]'),battery('R[2]')];
            obj.T0     = battery('T[0]');
            obj.T50    = battery('T[50]');
            obj.Tbeta  = battery('T[beta]');
            obj.cpB    = battery('c[pB]');
            obj.mB     = battery('m[B]');             
            % derived properties
            obj.hB     = log(2)*obj.mB*obj.cpB/obj.T50;                    
        end
                
        function rhs = rightHandSides(obj, d, f)
            % computes the right-hand-sides of 

            % battery currents
            IB = d.pedal(f)*f.IM;     
            I2 = f.UB2/obj.R(2);
            IC  = IB-I2;
            
            % state of depletion
            SD = 1-f.QB/obj.QB0;
            % battery potential voltage
            e = obj.c(1)+obj.c(2)*SD+obj.c(3)*SD^2+obj.c(4)*SD^3+obj.c(5)*SD^4+obj.c(6)*SD^5;
            UE = obj.U0*e;
            % voltage over resistances
            U1 = obj.R(1)*IB;
            U2 = obj.R(2)*I2;
            f.UB = U1+U2+UE;
            
            % heats
            QpH = obj.R(1)*IB^2 + obj.R(2)*I2^2;
            QpD = obj.hB*(f.TB-obj.T0);
                       
            rhs = [...
                -(1+IB/obj.Ialpha)*(1-f.TB/obj.Tbeta)*IB;...
                  IC/obj.C;...
                  1./(obj.mB*obj.cpB)*(QpH-QpD)];  
        end
    end
end

