classdef cars < handle
    % car class
    % delivers the dydt 
    properties
        a
        R
        k
        b
        m
        J
        M
        invM
        g
    end
    
    methods
        function obj = cars(car)
            % constructor class
            % read hashMap
            obj.a  = [car('a[1]'),car('a[2]'),car('a[3]')];
            obj.R  = car('R');
            obj.b  = [car('b[1]'),car('b[2]')];
            obj.k  = [car('k[1]'),car('k[2]')];
            obj.m  = [car('m[1]'),car('m[2]')];
            obj.g  =  car('g');
            obj.J  = [car('J[1]'),car('J[2]')];

            if obj.k(2) == 0
                % choose, so that phi is zero when the car rests upon a
                % plane surface
                a0 = obj.a(1)+obj.a(2);
                obj.k(2) = obj.k(1) * (a0*obj.m(2)+obj.a(1)*obj.m(1))/(a0*obj.m(2)+(obj.a(2))*obj.m(1));
                obj.k(2) = obj.k(1) * obj.a(1)/obj.a(2);
            end
        end
        
        function initialize(obj)
            % compute missing properties
            % compute mass matrix and its inverse
            obj.M = zeros(6,6);
            obj.M( 1 , 1 ) =  2*obj.m(2)+obj.m(1);
            obj.M( 1 , 5 ) =  2*obj.m(2)*obj.a(3);
            obj.M( 2 , 2 ) =  2*obj.m(2)+obj.m(1); 
            obj.M( 2 , 3 ) =  obj.m(2); 
            obj.M( 2 , 4 ) =  obj.m(2); 
            obj.M( 2 , 5 ) =  obj.a(1)*obj.m(2)-obj.a(2)*obj.m(2) ;
            obj.M( 3 , 3 ) =  obj.m(2); 
            obj.M( 3 , 5 ) =  obj.a(1)*obj.m(2); 
            obj.M( 4 , 4 ) =  obj.m(2); 
            obj.M( 4 , 5 ) =  -obj.a(2)*obj.m(2); 
            obj.M( 5 , 5 ) =  2*obj.m(2)*obj.a(3)^2+obj.a(2)^2*obj.m(2)+obj.a(1)^2*obj.m(2)+obj.J(1); 
            obj.M( 6 , 6 ) =  obj.J(2);
            % make ist symmetric
            for row=2:6
                for col=1:row-1
                    obj.M(row,col)=obj.M(col,row);
                end
            end
            % compute the inverse 
            % obj.invM = linsolve(obj.M,eye(6,6)); -> must be done in
            % drive-train after motor was added
        end
            
        function rhs = rightHandSides(obj, y, MW, FN, FT, singamma, cosgamma, gamma)
            % computes the right-hand-sides of 
            %   M dydt = rhs

            offset = 3;
            v(1) = y(offset+ 1);
            v(2) = y(offset+ 2);
            v(3) = y(offset+ 3);
            v(4) = y(offset+ 4);
            phiP = y(offset+ 5);
            omega= y(offset+ 6);

            u(1) = y(offset+ 7);
            u(2) = y(offset+ 8);
            u(3) = y(offset+ 9);
            u(4) = y(offset+10);
            phi  = y(offset+11);
            
            sinphi = sin(phi);
            cosphi = cos(phi);
            
            sindelta(1) = sin(phi-gamma(1));
            cosdelta(1) = cos(phi-gamma(1));
            sindelta(2) = sin(phi-gamma(2));
            cosdelta(2) = cos(phi-gamma(2));
            
            rhs( 1,1 ) =  cosgamma(1)*FT-singamma(2)*FN(2)-singamma(1)*FN(1);
            %--------------------------------------------------- 
            rhs( 2,1 ) =  ((-2*obj.m(2))-obj.m(1))*obj.g+singamma(1)*FT+cosgamma(2)*FN(2)+cosgamma(1)*FN(1);
            %--------------------------------------------------- 
            rhs( 3,1 ) = (-FT*sindelta(1))+FN(1)*cosdelta(1)-obj.m(2)*obj.g*cosphi-obj.b(1)*v(3)-obj.k(1)*u(3);
            %--------------------------------------------------- 
            rhs( 4,1 ) =                   FN(2)*cosdelta(2)-obj.m(2)*obj.g*cosphi-obj.b(2)*v(4)-obj.k(2)*u(4);
            %---------------------------------------------------             
            rhs( 5,1 ) =  obj.a(3)*FN(2)*sindelta(2)-obj.a(2)*FN(2)*cosdelta(2)...
                         +(obj.a(3)*FN(1)-obj.a(1)*FT)*sindelta(1) ...
                         +(obj.a(3)*FT+obj.a(1)*FN(1))*cosdelta(1) ...
                         +((obj.m(2)*obj.g-cosgamma(2)*FN(2))*u(4) ...
                         +(obj.m(2)*obj.g-singamma(1)*FT-cosgamma(1)*FN(1))*u(3) ...
                         -2*obj.m(2)*obj.a(3)*obj.g)*sinphi ...
                         +(singamma(2)*FN(2)*u(4) ...
                         +(singamma(1)*FN(1)-cosgamma(1)*FT)*u(3) ...
                         +(obj.a(2)-obj.a(1))*obj.m(2)*obj.g)*cosphi ...
                         +obj.R*FT+MW;
            %--------------------------------------------------- 
            rhs( 6,1 ) =  MW-obj.R*FT;

        end
    end
end

