classdef characteristics < handle
    % defines a characterisitc
    % made from line-sections with rounded edges 
    % works for any number of points
    
    properties
        x    % abszissa
        y    % ordinate
        eps  % rounding radius "Delta x"
        poly 
        nos
        title
        abscissaName 
        ordinateName 
        mode = 1 % "0" is the test mode with mode output displayed
    end
    
    methods
        function obj = characteristics(x,y,eps,title,abscissa,ordinate)
            % constructor class for 
            % this characteristic
            obj.title = title;
            obj.abscissaName = abscissa;
            obj.ordinateName = ordinate;
            
            % catch basic errors
            assert(size(x,1)==size( y ,1)&&size(x,2)  ==size( y ,2),"ERROR: x and y must be row matrices and their sizes must be identical");
            assert(size(x,1)==size(eps,1)&&size(x,2)-2==size(eps,2),"ERROR: sizes of eps mus match x / y sizes");
            obj.nos = 2*size(x,2)-3; % number of sections
         
            obj.x   = [x(1)];
            obj.y   = [y(1)];
            obj.poly= [];
           
            %declare line-polynomials
            for relay=1:size(x,2)-1
                a = (y(relay+1)-y(relay))/(x(relay+1)-x(relay));% inclination
                y0= y(relay)-a*x(relay);%
                polynomial = [0,a,y0]; % polynomial first order
                obj.poly = [obj.poly;polynomial;[0 0 0]];
            end
            obj.poly(2*(size(x,2)-1),:) = [];
            
            % declare x/y tuples at intersections of sections
            for relay=1:size(eps,2)
                X = [x(1+relay)-eps(relay),x(1+relay)+eps(relay)];
                obj.x   = [obj.x, X];
                obj.y   = [obj.y, [polyval(obj.poly(2*relay-1,:),X(1)),...
                                   polyval(obj.poly(2*relay+1,:),X(2))]];
            end
            
            % and now the missing polynomials
            % p = [a[2],a[1],a[0]]
            % p [x[i-1]] = y[i-1]
            % p'[x[i-1]] = obj.poly()
            for relay=1:size(eps,2)
                X = x(relay+1);
                Y = y(relay+1);
                S = [obj.poly(2*relay-1,2), obj.poly(2*relay+1,2)]; %inclination
                % phrase OLE <- these lines come from MAXIMA
                % P : [[a[1,1],a[1,0]],[a[2,1],a[2,0]]];
                % p : [                a[1,1]*x+a[1,0],
                %      b[3]*x^3+b[2]*x^2+b[1]*x+b[0],
                %                      a[2,1]*x+a[2,0]];
                % linear : solve([subst([x=x[1]],p[1]) = y[1],
		        %                 subst([x=x[1]],p[3]) = y[1]], [a[1,0],a[2,0]])[1];
                % equs : [subst([x=x[1]-eps], p[1]) = subst([x=x[1]-eps], p[2]),
                % subst([x=x[1]+eps], p[3]) = subst([x=x[1]+eps], p[2]),
                % subst([x=x[1]-eps], diff(p[1],x)) = subst([x=x[1]-eps], diff(p[2],x)),
                % subst([x=x[1]+eps], diff(p[3],x)) = subst([x=x[1]+eps], diff(p[2],x))];
                % Q : [b[3],b[2],b[1],b[0]];
                % sol : ratsimp(subst(linear,solve(equs,Q)));
                %
                % b[3]=0,
                % b[2]=(a[2,1]-a[1,1])/(4*eps),
                % b[1]=((a[2,1]+a[1,1])*eps-x[1]*a[2,1]+x[1]*a[1,1])/(2*eps),
                % b[0]=((a[2,1]-a[1,1])*eps^2+(-2*x[1]*a[2,1]-2*x[1]*a[1,1]+4*y[1])*eps+x[1]^2*a[2,1]-x[1]^2*a[1,1])/(4*eps)
                b2 = (S(2)-S(1))/(4*eps(relay));
                b1 = ((S(2)+S(1))*eps(relay)-X(1)*S(2)+X(1)*S(1))/(2*eps(relay));
                b0 = ((S(2)-S(1))*eps(relay)^2+(-2*X(1)*S(2)-2*X(1)*S(1)+4*Y(1))*eps(relay)+X(1)^2*S(2)-X(1)^2*S(1))/(4*eps(relay));
                obj.poly(2*relay,:) = [b2,b1,b0];
            end
                        
            obj.x   = [obj.x, x(1,size(x,2))];
            obj.y   = [obj.y, y(1,size(x,2))];
        
            obj.eps = eps;            
            
        end
        
        function [p,q] = getInterpolatedValues(obj,sectionNo)
            % returns plot values per section
            polynomial = obj.poly(sectionNo,:);
            X = [obj.x(sectionNo),obj.x(sectionNo+1)];
            if polynomial(1,2)==0
                % line
                p = X;                 
            else
                % parable
                p = X(1):(X(2)-X(1))/10:X(2);                 
            end
            q = polyval(polynomial,p);
        end
        
        function f = plots(obj)
            % plots the lines and parabolas
            f = figure('Name',['Characteristic',' ',obj.title],'NumberTitle','off');
            hold on;
            for section=1:obj.nos
                [p,q] = obj.getInterpolatedValues(section);
                plot(p,q);
            end
        end

        function setMode(obj,mode)
            assert(any([0 1]==mode),"ERROR: mode can only be 0 (test mode) or 1 (production mode)");
            obj.mode = mode;
        end

        function y = getValue(obj,x)
            % returns the ordinate(y)-value for the absizza(x)-value
            
            sectionID = 1;
            % find the right section "s" where x(s) < x < x(s+1)
            while sectionID < obj.nos
                if x > obj.x(sectionID+1)
                    sectionID = sectionID + 1;
                else
                    if obj.mode==0
                        % test mode
                        disp( ['sectionID = ',num2str(sectionID)]);
                    end
                    break;
                end
            end
            y = polyval(obj.poly(sectionID,:),x);            
        end
    end
end
