classdef controls < handle
    % Drive-Train class
    
    properties
        Kd
        Ki
        Kp
        TBmax
        Vset
        vCTarget
    end
    
    methods
        function obj = controls(driver)
            % constructs the class
            % 
            obj.Kd   = driver('K[d]');
            obj.Ki   = driver('K[i]');
            obj.Kp   = driver('K[p]');
            obj.TBmax= driver('T[Bmax]');
            obj.Vset = driver('V[set]');
        end
        
        function initialize(obj, f)
           % compute target slip velocity
           muC = f.muC.poly(6,:);
           obj.vCTarget = roots(polyder(muC));
        end
        
        function e = errorfct(obj,f)
            %compute error value
            % e(1) is deviation from optimal grip
            % e(2) is deviation from optimal speed
            e = [1 + f.vC(1,1)/obj.vCTarget,...
                 1 - f.tangentialSpeed()/obj.Vset];
        end
        
        function p = pedal(obj,f)
            % returns the pedel position
            err = min(errorfct(obj,f));
            p = obj.Kp*err + obj.Ki*f.E + 0;            
            % limit p to "1"
            p = max(0,min(p,1.0));
        end
    end
end

