classdef driveTrains < handle
    % Drive-Train class
    
    properties
        omegaNL
        MSt
        ISt
        L
        R
        UMref
        kT
        ke
        nG  
        J
    end
    
    methods
        function obj = driveTrains(driveParams)
            % constructs the class
            % 
            obj.MSt   = driveParams('M[St]');
            obj.ISt   = driveParams('I[St]');
            obj.omegaNL = driveParams('omega[NL]');
            obj.L     = driveParams('L');
            obj.UMref = driveParams('U[M]');
            obj.nG    = driveParams('n[G]');
            obj.J     = driveParams('J');
            % derived properties
            obj.kT = obj.MSt/obj.ISt;
            obj.ke = obj.UMref/obj.omegaNL;
            obj.R  = obj.UMref/obj.ISt;            
            
        end

        function initialize(obj)
            % compute missing properties
            %obj.kT = driveParams('k[T]');
            %obj.Ke = driveParams('k[e]');
        end
        
        function T = torque(obj,f)
            % returns the torque of the gear-box (from motor)
            % simple brushed-motor model
       
            T = obj.nG*obj.kT*f.IM;

        end
        
        function rhs = rightHandSides(obj,d,f)
            % computes the right-hand-sides of 
            % dI[M]dt = rhs
            p = d.pedal(f);                   
            rhs = 1/obj.L*(p*f.UB-obj.R*f.IM - obj.ke*obj.nG*f.omega);
            
        end
        
        function addContributionToMassMatrix(obj, c)
            % adds the elements from motor and gear-box to the total 
            % mass matrix M of the car
            n = obj.nG;
            addedMass = obj.J*[  n^2+2*n+1,-(n^2+n);...
                               -(n^2+n)   ,   n^2  ];
            c.M(5:6,5:6) = c.M(5:6,5:6)+addedMass;            
            % compute the inverse 
            c.invM = linsolve(c.M,eye(6,6));           
        end
        
        
    end
end

