function dim = dimensions()
% returns the value of the parameter as a number 
% made dimensionless
% reference dimensions
L = 1; %*m
T = 1; %*s
M = 1; %*kg 
C = 1; %*A (Ampere)
H = 1; %*K (Temperature)

F = M*L/T^2; % force
E = F*L;
P = E/T;   % power

U = P/C;     % voltage, https://en.wikipedia.org/wiki/Volt

dim = containers.Map('KeyType','char','ValueType','double');
dim('1')       = 1.;
dim('m')       = 1./L;
dim('kg')      = 1./M;
dim('s')       = 1./T;
dim('1/s')     = 1./(1/T);
dim('1/s^2')   = 1./(1/T^2);
dim('m/s')     = 1./(L/T);
dim('rad/s')   = 1./(1/T);
dim('kg*m^2')  = 1./(M*L^2);
dim('m/s^2')   = 1./(M/T^2);
dim('N')       = 1./(M*L/T^2);
dim('N/m')     = 1./(M*L/T^2/L);
dim('N*m')     = 1./(M*L/T^2*L);
dim('N/(m/s)') = 1./(M*L/T^2/(L/T));
dim('J/(kg*K)')= 1./(E/(M*H));


dim('N*m/V')   = 1./(F*L/U);
dim('N*m/A')   = 1./(F*L/C);
dim('V*s')     = 1./(U*T);

dim('A')       = 1./(C); 
dim('Ω')       = 1./(M*L^2/(T^3*C^2)); %https://en.wikipedia.org/wiki/Ohm
dim('H')       = 1./(M*L^2/(T^2*C^2)); %https://en.wikipedia.org/wiki/Henry_(unit)
dim('F')       = 1./(E/U^2); %https://en.wikipedia.org/wiki/Farad
dim('V')       = 1./(U); 
dim('Ah')      = 1./(C/(3600*T)); 

dim('°C')      = 1./(H); 

end

