function postpro(t,y,b,c,d,e,f)

%% post-process

eta  = zeros(size(y,1),2);
F    = zeros(size(y,1),3);
u    = zeros(size(y,1),4);
U    = zeros(size(y,1),4);
u    = zeros(size(y,1),4);
v    = zeros(size(y,1),4);
M    = zeros(size(y,1),1);
IM   = zeros(size(y,1),1);
phi  = zeros(size(y,1),1);
phiP = zeros(size(y,1),1);
omega= zeros(size(y,1),1);
QB   = zeros(size(y,1),1);
UB2  = zeros(size(y,1),1);
TB   = zeros(size(y,1),1);
eVal = zeros(size(y,1),2);
EVal = zeros(size(y,1),1);
pVal = zeros(size(y,1),1);
vT   = zeros(size(y,1),1);

for row=1:size(y,1)
    f.putY(y(row,:));
    for axle = 1:2
        xi = f.getContactGeometry(axle);
        if axle == 1
            [FN(axle), FT] = f.contactF(axle);
        else
            [FN(axle), ~] = f.contactF(axle);
        end
    end
    
    QB(row,1)   = f.QB;
    UB2(row,1)  = f.UB2;
    TB(row,1)   = f.TB;

    eta(row,:)  = f.eta;
    F(row,:)    = [FN,FT];   
    U(row,:)    = f.U(3:6);   
    u(row,:)    = f.u;   
    v(row,:)    = f.v;
    vT(row,:)   = f.tangentialSpeed();
    phi(row,:)  = f.phi;
    phiP(row,:) = f.phiP;
    omega(row,:)= f.omega;

    eVal(row,:) = d.errorfct(f);
    EVal(row,:) = f.E;
    pVal(row,:) = d.pedal(f);
    
    M(row,1)    = e.torque(f);   
    IM(row,1)   = f.IM;   
end

%% plot
tiledlayout(3,3);

% Top Left ----------------------------------
nexttile;
title('body-coordinates (center of mass)');
hold on;
grid on;
for i=[1,2]
    plot(t,u(:,i))
end
legend('u_1','u_2')
hold off;
xlabel('t/s →')
ylabel('u/m →')
% Top Center -----------------------
nexttile;
title('suspension deflection / body rotation angle');
hold on;
grid on;
for i=[3,4]
    plot(t,u(:,i)/f.a(3))
end
for i=[1]
    plot(t,phi(:,i))
end
legend('u_3/a_3','u_4/a_3','\phi');
xlabel('t/s →')
ylabel('u_i,\phi →')

% Top Right -----------------------
nexttile;
title('relative track penetration');
hold on;
grid on;
plot(t,eta(:,1)-1)
plot(t,eta(:,2)-1)
legend('axle 1', 'axle 2');
xlabel('t/s →')
ylabel('\eta_i-1 →')
% Middle Left -----------------------
nexttile;
title('body and wheel spatial trajectories');
hold on;
grid on;
p=[];
q=[];
% Track
for sectionID=1:f.trackC.nos
    [r,s] = f.trackC.getInterpolatedValues(sectionID);
    p = [p, r];
    q = [q, s];
end
plot(p,q);
% Center Point
plot(u(:,1),u(:,2));
plot(U(:,1),U(:,2));
plot(U(:,3),U(:,4));
hold off;
legend('track','car body','front wheel','rear wheel','location','northwest');
xlabel('x/m →')
ylabel('y/m →')

% Middle Center -----------------------
nexttile;
title('Contact Forces, torque on wheel, Motor current');
grid on;
hold on;
for i=1:3
    plot(t,F(:,i)/(c.g*c.m(1)));
end
plot(t,M/(c.g*c.m(1)*c.R));
plot(t,IM/e.ISt);
hold off;

legend('F_{N1}','F_{N2}','F_T','M_M','I_M');
xlabel('t/s →')
ylabel('F_i/G, M_W/R·G, I_M/I_{St} →')
hold off;

% Middle Right -----------------------
nexttile
title('velocities');
hold on;
grid on;
scale = [0 1 1 1 1/c.R];
for i=2:4
    plot(t,v(:,i)/scale(i));
end
for i=5:5
    plot(t,phiP/scale(i));
end

legend('v_2','v_3','v_4','R d\phi/dt','location','southeast');
xlabel('t/s →')
ylabel('v/(m/s) →')

hold off;

% Bottom Left -----------------------
nexttile
v0=d.Vset;
title(strcat('velocities, v_0=', num2str(v0),'m/s'));
hold on;
grid on;
scale = [v0, v0/c.R];
plot(t,v( :,1)/scale(1));
plot(t,vT(:,1)/scale(1));
plot(t,omega/scale(2));

legend('v_1/v_0','v_T/v_0', '\omega/\omega_0','location','southeast');
xlabel('t/s →')
ylabel('v/v_0 →')

hold off;

% Bottom Center -----------------------
nexttile;
title('battery');
hold on;
grid on;

plot(t,QB/b.QB0);
plot(t,UB2/b.U0);
plot(t,TB/d.TBmax);
hold off;
legend('Q_B/Q_{B0}','U_{B2}/U_0','T_B/T_{max}','location','northwest');
xlabel('t/s →')
ylabel('Q_B,U_{B2},T_B →')

% Bottom Right -----------------------
nexttile;
title('driver controls');
hold on;
grid on;

plot(t,eVal(:,1));
plot(t,eVal(:,2));
plot(t,EVal);
plot(t,pVal);
hold off;
legend('e_1','e_2','E','p','location','northwest');
xlabel('t/s →')
ylabel('e,E,p →')

end

